<?php

namespace Drupal\drucash\Controller;

use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityListBuilder;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Entity\Query\QueryInterface;
use Drupal\Core\Routing\RouteMatchInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Builds the transaction list for a specific account.
 */
class AccountLedgerListBuilder extends EntityListBuilder {

  /**
   * The route match service.
   *
   * @var \Drupal\Core\Routing\RouteMatchInterface
   */
  protected RouteMatchInterface $routeMatch;

  /**
   * {@inheritdoc}
   */
  public function __construct(EntityTypeInterface $entity_type, EntityStorageInterface $storage, RouteMatchInterface $route_match) {
    parent::__construct($entity_type, $storage);
    $this->routeMatch = $route_match;
  }

  /**
   * {@inheritdoc}
   */
  public static function createInstance(ContainerInterface $container, EntityTypeInterface $entity_type) {
    return new static(
      $entity_type,
      $container->get('entity_type.manager')->getStorage($entity_type->id()),
      $container->get('current_route_match')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function buildHeader(): array {
    $header = [];
    $header['id'] = $this->t('ID');
    $header['date'] = $this->t('Date');
    $header['name'] = $this->t('Name');
    $header['amount'] = $this->t('Amount');
    $header['balance'] = $this->t('Balance');
    return $header + parent::buildHeader();
  }

  /**
   * {@inheritdoc}
   */
  public function buildRow(EntityInterface $entity): array {
    /** @var \Drupal\drucash\Entity\Transaction $entity */

    $date = $entity->getDate();

    $row = [];
    $row['id'] = $entity->id();
    $row['date'] = $date ? $date->format('d/m/Y') : '';
    $row['name'] = $entity->label();
    $row['amount'] = $entity->getAmount();
    $row['balance'] = '0.0';
    return $row + parent::buildRow($entity);
  }

  /**
   * Generates the query to match transactions affecting the account.
   */
  protected function getEntityListQuery(): QueryInterface {

    // Get the account ID from the url.
    $account_id = $this->routeMatch->getParameter('drucash_account');

    $query = $this->getStorage()->getQuery()
      ->accessCheck(TRUE)
      ->sort('date');

    $group = $query->orConditionGroup();
    $group->condition('from', $account_id);
    $group->condition('to', $account_id);

    $query->condition($group);

    // Only add the pager if a limit is specified.
    if ($this->limit) {
      $query->pager($this->limit);
    }

    return $query;
  }

}
