<?php

namespace Drupal\drucash\Entity;

use Drupal\Core\Entity\ContentEntityBase;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Field\BaseFieldDefinition;
use Drupal\Core\Field\FieldStorageDefinitionInterface;

/**
 * Defines the Account entity.
 *
 * @ingroup drucash
 *
 * @ContentEntityType(
 *   id = "drucash_account",
 *   label = @Translation("Account"),
 *   label_collection = @Translation("Accounts"),
 *   label_singular = @Translation("account"),
 *   label_plural = @Translation("accounts"),
 *   label_count = @PluralTranslation(
 *     singular = "@count account",
 *     plural = "@count accounts",
 *   ),
 *   base_table = "drucash_account",
 *   admin_permission = "administer drucash_account",
 *   handlers = {
 *     "list_builder" = "Drupal\drucash\Controller\AccountListBuilder",
 *     "form" = {
 *       "add" = "Drupal\drucash\Form\AccountForm",
 *       "edit" = "Drupal\drucash\Form\AccountForm",
 *       "delete" = "Drupal\Core\Entity\ContentEntityDeleteForm",
 *       "delete-multiple-confirm" = "Drupal\Core\Entity\Form\DeleteMultipleForm",
 *     },
 *     "route_provider" = {
 *       "html" = "Drupal\Core\Entity\Routing\AdminHtmlRouteProvider"
 *     },
 *   },
 *   links = {
 *     "collection" = "/admin/content/account",
 *     "add-form" = "/account/add",
 *     "canonical" = "/account/{drucash_account}",
 *     "edit-form" = "/account/{drucash_account}/edit",
 *     "delete-form" = "/account/{drucash_account}/delete",
 *     "delete-multiple-form" = "/admin/content/account/delete-multiple",
 *   },
 *   entity_keys = {
 *     "id" = "id",
 *     "uuid" = "uuid",
 *   },
 * )
 */
class Account extends ContentEntityBase implements AccountInterface {

  /**
   * {@inheritdoc}
   */
  public static function baseFieldDefinitions(EntityTypeInterface $entity_type): array {

    $fields = parent::baseFieldDefinitions($entity_type);

    $fields['name'] = BaseFieldDefinition::create('string')
      ->setLabel('Name')
      ->setDescription('Account name')
      ->setRequired(TRUE)
      ->setDefaultValue('')
      ->setDisplayOptions('form', [
        'type' => 'string',
        'weight' => 1.0,
      ]);

    $fields['description'] = BaseFieldDefinition::create('string_long')
      ->setLabel('Description')
      ->setDescription('Account description')
      ->setDefaultValue('')
      ->setDisplayOptions('form', [
        'type' => 'basic_string',
        'weight' => 3.0,
      ]);

    $fields['type'] = BaseFieldDefinition::create('list_string')
      ->setLabel('Type')
      ->setDescription('Account type')
      ->setRequired(TRUE)
      ->setSettings([
        'allowed_values' => [
          'assets' => t('Assets'),
          'income' => t('Income'),
          'expenses' => t('Expenses'),
        ],
      ])
      ->setDisplayOptions('form', [
        'type' => 'list_default',
        'weight' => 2.0,
      ]);

    $fields['children'] = BaseFieldDefinition::create('entity_reference')
      ->setLabel('Children')
      ->setDescription('Children accounts')
      ->setRequired(FALSE)
      ->setCardinality(FieldStorageDefinitionInterface::CARDINALITY_UNLIMITED)
      ->setDefaultValue([])
      ->setSettings([
        'target_type' => 'drucash_account',
      ]);

    return $fields;
  }

  /**
   * {@inheritdoc}
   */
  public function label() {
    return $this->getName();
  }

  /**
   * {@inheritdoc}
   */
  public function getName(): string {
    return $this->get('name')->isEmpty() ? '' : $this->get('name')->first()->getString();
  }

  /**
   * {@inheritdoc}
   */
  public function getDescription(): string {
    return $this->get('description')->isEmpty() ? '' : $this->get('description')->first()->getString();
  }

  /**
   * {@inheritdoc}
   */
  public function getType(): string {
    return $this->get('type')->isEmpty() ? '' : $this->get('type')->first()->getString();
  }

  /**
   * {@inheritdoc}
   */
  public function getTypeLabel() {
    $value = $this->get('type')->first();
    $allowed_values = $value->getDataDefinition()->getSetting('allowed_values');
    return $allowed_values[$value->getString()] ?? '';
  }

  /**
   * {@inheritdoc}
   */
  public function getChildren(): array {
    return $this->get('children')->getValue();
  }

  /**
   * {@inheritdoc}
   */
  public function getBalance(): float {

    $ids = array_merge([$this->id()], $this->getChildren());
    $balance = 0.0;

    $input = \Drupal::database()->select('drucash_transaction');
    $input->addExpression('SUM(amount)', 'total');
    $input->condition('to', $ids, 'IN');
    $input_sum = $input->execute()->fetch(\PDO::FETCH_ASSOC);
    if ($input_sum['total']) {
      $balance += $input_sum['total'];
    }

    $output = \Drupal::database()->select('drucash_transaction');
    $output->addExpression('SUM(amount)', 'total');
    $output->condition('from', $ids, 'IN');
    $output_sum = $output->execute()->fetch(\PDO::FETCH_ASSOC);
    if ($output_sum['total']) {
      $balance -= $output_sum['total'];
    }

    return $balance;
  }

  /**
   * {@inheritdoc}
   */
  public function setName(string $name): void {
    $this->set('name', $name);
  }

  /**
   * {@inheritdoc}
   */
  public function setDescription(string $description): void {
    $this->set('description', $description);
  }

  /**
   * {@inheritdoc}
   */
  public function setType(string $type): void {
    $this->set('type', $type);
  }

  /**
   * {@inheritdoc}
   */
  public function addChildren(AccountInterface $child): void {
    $children_ids = $this->get('children')->getValue();
    $ids = array_column($children_ids, 'target_id');
    if (!in_array($child->id(), $ids)) {
      $this->get('children')->appendItem(['target_id' => $child->id()]);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function removeChildren(AccountInterface $child): void {
    $children_ids = $this->get('children')->getValue();
    $key = array_search($child->id(), array_column($children_ids, 'target_id'));
    if ($key !== FALSE) {
      $this->get('children')->removeItem($key);
    }
  }

}
