<?php

namespace Drupal\drucash\Form;

use Drupal\Component\Datetime\TimeInterface;
use Drupal\Core\Entity\ContentEntityForm;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityRepositoryInterface;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Entity\EntityTypeBundleInfoInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\drucash\Entity\Account;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Account entity creation form.
 */
class AccountForm extends ContentEntityForm {

  /**
   * The account entity storage.
   *
   * @var \Drupal\Core\Entity\EntityStorageInterface
   */
  protected EntityStorageInterface $entityStorage;

  /**
   * {@inheritdoc}
   */
  public function __construct(EntityRepositoryInterface $entity_repository, EntityTypeBundleInfoInterface $entity_type_bundle_info, TimeInterface $time, EntityStorageInterface $entity_storage) {
    parent::__construct($entity_repository, $entity_type_bundle_info, $time);
    $this->entityStorage = $entity_storage;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity.repository'),
      $container->get('entity_type.bundle.info'),
      $container->get('datetime.time'),
      $container->get('entity_type.manager')->getStorage('drucash_account'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildForm($form, $form_state);

    $form['parent'] = [
      '#type' => 'entity_autocomplete',
      '#title' => $this->t('Parent account'),
      '#target_type' => 'drucash_account',
      '#weight' => 1.5,
    ];

    $entity = $this->getEntity();
    $parent = $this->getAccountParent($entity);
    if ($parent) {
      $form['parent']['#default_value'] = $parent;
    }

    return $form;
  }

  /**
   * Adds or removes the account to the parent one accordingly.
   */
  public function submitForm(&$form, FormStateInterface $form_state) {

    /** @var \Drupal\drucash\Entity\AccountInterface $entity */
    $entity = $this->getEntity();

    // Remove any existing parents.
    if (!$entity->isNew()) {
      $query = $this->entityStorage->getQuery()
        ->accessCheck(FALSE)
        ->condition('children', $entity->id(), 'CONTAINS');

      $parents = $query->execute();
      foreach ($parents as $parent_id) {
        $parent = Account::load($parent_id);
        $parent->removeChildren($entity);
        $parent->save();
      }
    }

    $parent_id = $form_state->getValue('parent');
    if ($parent_id) {
      $parent = Account::load($parent_id);
      $parent->addChildren($entity);
      $parent->save();
    }

    parent::submitForm($form, $form_state);

    $form_state->setRedirect('entity.drucash_account.collection');
  }

  /**
   * Get the parent account for an account.
   */
  protected function getAccountParent(EntityInterface $entity): ?Account {

    if ($entity->isNew()) {
      return NULL;
    }

    // Check if any account has the entity listed in its children field.
    $query = $this->entityStorage->getQuery()
      ->accessCheck(FALSE)
      ->condition('children', $entity->id(), 'CONTAINS');

    $result = $query->execute();
    if (empty($result)) {
      return NULL;
    }

    $item_id = array_shift($result);
    return Account::load($item_id);
  }

}
