<?php

declare(strict_types = 1);

namespace Drupal\Tests\drucash\Kernel;

use Drupal\drucash\Entity\Account;
use Drupal\drucash\Entity\Transaction;
use Drupal\KernelTests\KernelTestBase;

/**
 * Tests the drucash_account entity.
 *
 * @group drucash
 */
final class AccountEntityTest extends KernelTestBase {

  /**
   * {@inheritdoc}
   */
  protected static $modules = ['drucash', 'options', 'datetime'];

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();
    $this->installEntitySchema('drucash_account');
    $this->installEntitySchema('drucash_transaction');
  }

  /**
   * Tests the drucash_account entity class.
   */
  public function testAccountCreate(): void {
    $account = Account::create([]);
    $this->assertInstanceOf(Account::class, $account);
  }

  /**
   * Tests validation without all required fields fail.
   */
  public function testValidateWithoutRequiredFields(): void {
    $account = Account::create([]);
    $this->assertNotEmpty($account->validate());
  }

  /**
   * Test basic account validation.
   */
  public function testBasicAccountValidation() {
    $account = Account::create([
      'name' => 'Account',
      'type' => 'assets',
    ]);
    $this->assertEmpty($account->validate());
  }

  /**
   * Tests getters and setters.
   */
  public function testGettersAndSetters() {
    $account = Account::create([]);
    $account->setName('Account');
    $account->setDescription('Description');
    $account->setType('assets');
    $this->assertEquals('Account', $account->getName());
    $this->assertEquals('Description', $account->getDescription());
    $this->assertEquals('assets', $account->getType());
  }

  /**
   * Assigning an invalid account type should fail.
   */
  public function testSetInvalidAccountType() {
    $account = Account::create([]);
    $account->setType('invalid');
    $this->assertNotEmpty($account->validate());
  }

  /**
   * By default, a new account has no parent one.
   */
  public function testDefaultAccountValues() {
    $account = Account::create([]);
    $this->assertEmpty($account->getChildren());
    $this->assertEquals(0.00, $account->getBalance());
  }

  /**
   * Tests setting and getting a children account.
   */
  public function testSetAccountChildren() {
    $parent = Account::create([]);

    $child = Account::create([]);
    $child->save();

    $parent->addChildren($child);
    $this->assertCount(1, $parent->getChildren());
  }

  /**
   * Adding the same child account twice does not duplicate it.
   */
  public function testSetAccountChildrenTwice() {
    $parent = Account::create([]);

    $child = Account::create([]);
    $child->save();

    $parent->addChildren($child);
    $parent->addChildren($child);

    $this->assertCount(1, $parent->getChildren());
  }

  /**
   * Tests adding and removing a children account.
   */
  public function testSetAccountChildrenAndRemove() {
    $parent = Account::create([]);

    $child = Account::create([]);
    $child->save();

    $parent->addChildren($child);
    $this->assertCount(1, $parent->getChildren());
    $parent->removeChildren($child);
    $this->assertCount(0, $parent->getChildren());
  }

  /**
   * Tests the balance ammount after adding transactions.
   */
  public function testBalanceAfterAddingTransactions() {

    $account = Account::create([]);
    $account->save();

    $account2 = Account::create([]);
    $account2->save();

    $trans1 = Transaction::create([]);
    $trans1->setAmount(20.3);
    $trans1->setFrom($account);
    $trans1->setTo($account2);
    $trans1->save();

    $trans2 = Transaction::create([]);
    $trans2->setAmount(10.3);
    $trans2->setFrom($account2);
    $trans2->setTo($account);
    $trans2->save();

    $this->assertEquals(-10, $account->getBalance());
    $this->assertEquals(10, $account2->getBalance());
  }

}
